@@ -44,6 +44,8 @@ public:
         if (d.isNull()) {
             d.reset(new KWindowSystemPrivateDummy());
         }
+
+        kwm.moveToThread(QCoreApplication::instance()->thread());
     }
     KWindowSystem kwm;
     QScopedPointer<KWindowSystemPrivate> d;
@@ -342,9 +344,9 @@ QList<WId> KWindowSystem::windows()
 }
 
 #ifndef KWINDOWSYSTEM_NO_DEPRECATED
-KWindowInfo KWindowSystem::windowInfo(WId win, unsigned long properties, unsigned long properties2)
+KWindowInfo KWindowSystem::windowInfo(WId win, NET::Properties properties, NET::Properties2 properties2)
 {
-    return KWindowInfo(win, NET::Properties(uint(properties)), NET::Properties2(uint(properties2)));
+    return KWindowInfo(win, properties, properties2);
 }
 #endif
 
