@@ -29,7 +29,6 @@
 #include <QWidgetList> //For WId
 #include <netwm_def.h>
 #include <kwindowinfo.h>
-#include <config-kwindowsystem.h>
 
 class KWindowSystemPrivate;
 
@@ -81,7 +80,7 @@ public:
       * \endcode
       * @return the list of all toplevel windows
       */
-    static const QList<WId> &windows();
+    static QList<WId> windows();
 
     /**
      * Test to see if @p id still managed at present.
@@ -90,6 +89,7 @@ public:
      **/
     static bool hasWId(WId id);
 
+#ifndef KWINDOWSYSTEM_NO_DEPRECATED
     /**
      * Returns information about window @p win. It is recommended to check
      * whether the returned info is valid by calling the valid() method.
@@ -99,8 +99,10 @@ public:
      *    in the returned data, but make this function faster when not all data is needed.
      * @param properties2 additional properties (see NET::Property2 enum)
      * @return the window information
+     * @deprecated use KWindowInfo directly
      */
-    static KWindowInfo windowInfo(WId win, unsigned long properties, unsigned long properties2 = 0);
+    KWINDOWSYSTEM_DEPRECATED static KWindowInfo windowInfo(WId win, NET::Properties properties, NET::Properties2 properties2 = 0);
+#endif
 
     /**
      * Returns the list of all toplevel windows currently managed by the
@@ -225,20 +227,22 @@ public:
      * that belongs to another application.
      */
     static void setMainWindow(QWidget *subwindow, WId mainwindow);
-#if HAVE_X11
+#ifndef KWINDOWSYSTEM_NO_DEPRECATED
     /**
      * Returns the WM_TRANSIENT_FOR property for the given window, i.e. the mainwindow
      * for this window.
      *
      * @param window the id of the window
+     * @deprecated Use KWindowInfo::transientFor
      */
-    static WId transientFor(WId window);
+    KWINDOWSYSTEM_DEPRECATED static WId transientFor(WId window);
 
     /**
      * Returns the leader window for the group the given window is in, if any.
      * @param window the id of the window
+     * @deprecated Use KWindowInfo::groupLeader
      */
-    static WId groupLeader(WId window);
+    KWINDOWSYSTEM_DEPRECATED static WId groupLeader(WId window);
 #endif
     /**
      * Returns an icon for window @p win.
@@ -312,7 +316,7 @@ public:
      * @param win the id of the window
      * @param state the new flags that will be set
      */
-    static void setState(WId win, unsigned long state);
+    static void setState(WId win, NET::States state);
 
     /**
      * Clears the state of window @p win from @p state.
@@ -325,7 +329,7 @@ public:
      * @param win the id of the window
      * @param state the flags that will be cleared
      */
-    static void clearState(WId win, unsigned long  state);
+    static void clearState(WId win, NET::States  state);
 
     /**
      * Iconifies a window. Compatible to XIconifyWindow but has an
@@ -491,7 +495,6 @@ public:
      */
     static void setBlockingCompositing(WId window, bool active);
 
-#if HAVE_X11
     /**
      * @internal
      * Returns true if viewports are mapped to virtual desktops.
@@ -519,7 +522,6 @@ public:
      * valid desktop area.
      */
     static QPoint constrainViewportRelativePosition(const QPoint &pos);
-#endif
 
 Q_SIGNALS:
 
@@ -576,6 +578,19 @@ Q_SIGNALS:
      */
     void stackingOrderChanged();
 
+    /**
+     * The window changed.
+     *
+     * Carries the NET::Properties and NET::Properties2 that were changed.
+     *
+     * @param id the id of the window
+     * @param properties the properties that were modified
+     * @param properties2 the properties2 that were modified
+     *
+     * @since 5.0
+     **/
+    void windowChanged(WId id, NET::Properties properties, NET::Properties2 properties2);
+
     /**
      * The window changed.
      *
@@ -585,8 +600,12 @@ Q_SIGNALS:
      * is the same like for the NETWinInfo class constructor).
      * @param id the id of the window
      * @param properties the properties that were modified
+     *
+     * @deprecated since 5.0 use windowChanged(WId, NET::Properties, NET::Properties2)
      */
-    void windowChanged(WId id, const unsigned long *properties);
+#ifndef KWINDOWSYSTEM_NO_DEPRECATED
+    QT_MOC_COMPAT void windowChanged(WId id, const unsigned long *properties);
+#endif
 
     /**
      * @deprecated
@@ -597,7 +616,9 @@ Q_SIGNALS:
      * @param id the id of the window
      * @param properties the properties that were modified
      */
-    void windowChanged(WId id, unsigned int properties);
+#ifndef KWINDOWSYSTEM_NO_DEPRECATED
+    QT_MOC_COMPAT void windowChanged(WId id, unsigned int properties);
+#endif
 
     /**
      * The window changed somehow.
@@ -631,15 +652,7 @@ private:
     friend class KWindowSystemStaticContainer;
 
     KWindowSystem() {}
-
-    enum { INFO_BASIC = 1, // desktop info, not per-window
-           INFO_WINDOWS = 2
-         }; // also per-window info
-
-    static void init(int);
-
-    friend class KWindowSystemPrivate;
-    static KWindowSystemPrivate *s_d_func();
+    static KWindowSystemPrivate *d_func();
 };
 
 #endif
